@@ -3,8 +3,8 @@
 //!
 //! Many of these functions are more or less broken and incomplete.
 
-use fish_wchar::prelude::*;
 use fish_widecharwidth::{WcLookupTable, WcWidth};
+use fish_widestring::prelude::*;
 use std::cmp;
 use std::sync::{
     LazyLock,
@@ -36,7 +36,10 @@ pub fn wcwidth(c: char) -> isize {
         pub unsafe fn wcwidth(c: libc::wchar_t) -> libc::c_int;
     }
 
-    const _: () = assert!(std::mem::size_of::<libc::wchar_t>() >= std::mem::size_of::<char>());
+    const {
+        assert!(size_of::<libc::wchar_t>() >= size_of::<char>());
+    }
+
     let width = unsafe { wcwidth(c as libc::wchar_t) };
     isize::try_from(width).unwrap()
 }
@@ -185,7 +188,7 @@ fn lowercase_impl<ToLowercase: Iterator<Item = char>>(
 #[cfg(test)]
 mod tests {
     use super::wcscasecmp;
-    use fish_wchar::prelude::*;
+    use fish_widestring::prelude::*;
     use std::cmp::Ordering;
 
     #[test]
