@@ -1,6 +1,6 @@
 use crate::arg::ToArg;
 use crate::locale::{C_LOCALE, EN_US_LOCALE, Locale};
-use crate::{Error, FormatString, sprintf_locale};
+use crate::{Error, FormatString as _, sprintf_locale};
 use std::f64::consts::{E, PI, TAU};
 use std::ffi::CStr;
 use std::fmt;
@@ -13,7 +13,7 @@ macro_rules! sprintf_check {
         $(,)? // optional trailing comma
     ) => {
         {
-            use unicode_width::UnicodeWidthStr;
+            use unicode_width::UnicodeWidthStr as _;
             let mut target = String::new();
             let mut args = [$($arg.to_arg()),*];
             let len = $crate::printf_c_locale(
@@ -861,8 +861,8 @@ fn test_float_hex_prec() {
     let mut libc_sprintf = libc_sprintf_one_float_with_precision(&mut c_storage, c"%.*a");
 
     let mut failed = false;
-    for sign in [1.0, -1.0].into_iter() {
-        for mut v in [0.0, 0.5, 1.0, 1.5, PI, TAU, E].into_iter() {
+    for sign in [1.0, -1.0] {
+        for mut v in [0.0, 0.5, 1.0, 1.5, PI, TAU, E] {
             v *= sign;
             for preci in 1..=200_usize {
                 rust_str.clear();
