@@ -1,5 +1,5 @@
 use fish::builtins::fish_key_reader;
 
 fn main() {
-    fish_key_reader::main()
+    fish_key_reader::main();
 }
