@@ -2,7 +2,7 @@
 
 use super::prelude::*;
 use crate::{
-    env::{EnvMode, Environment},
+    env::{EnvMode, Environment as _},
     fds::{BEST_O_SEARCH, wopen_dir},
     parser::ParserEnvSetMode,
     path::path_apply_cdpath,
@@ -10,11 +10,17 @@ use crate::{
 };
 use errno::Errno;
 use libc::{EACCES, ELOOP, ENOENT, ENOTDIR, EPERM};
-use std::{os::fd::AsRawFd, sync::Arc};
+use nix::unistd::fchdir;
+use std::sync::Arc;
 
 // The cd builtin. Changes the current directory to the one specified or to $HOME if none is
 // specified. The directory can be relative to any directory in the CDPATH variable.
 pub fn cd(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
+    localizable_consts! {
+        DIR_DOES_NOT_EXIST
+        "%s: The directory '%s' does not exist"
+    }
+
     let Some(&cmd) = args.first() else {
         return Err(STATUS_INVALID_ARGS);
     };
@@ -40,7 +46,7 @@ pub fn cd(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
             None => {
                 streams
                     .err
-                    .append(&wgettext_fmt!("%s: Could not find home directory\n", cmd));
+                    .appendln(&wgettext_fmt!("%s: Could not find home directory", cmd));
                 return Err(STATUS_CMD_ERROR);
             }
         }
@@ -48,14 +54,14 @@ pub fn cd(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
 
     // Stop `cd ""` from crashing
     if dir_in.is_empty() {
-        streams.err.append(&wgettext_fmt!(
-            "%s: Empty directory '%s' does not exist\n",
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: Empty directory '%s' does not exist",
             cmd,
             dir_in
         ));
         if !parser.is_interactive() {
             streams.err.append(&parser.current_line());
-        };
+        }
         return Err(STATUS_CMD_ERROR);
     }
 
@@ -63,11 +69,9 @@ pub fn cd(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
 
     let dirs = path_apply_cdpath(dir_in, &pwd, vars);
     if dirs.is_empty() {
-        streams.err.append(&wgettext_fmt!(
-            "%s: The directory '%s' does not exist\n",
-            cmd,
-            dir_in
-        ));
+        streams
+            .err
+            .appendln(&wgettext_fmt!(DIR_DOES_NOT_EXIST, cmd, dir_in));
 
         if !parser.is_interactive() {
             streams.err.append(&parser.current_line());
@@ -88,10 +92,11 @@ pub fn cd(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
         let res = wopen_dir(&norm_dir, BEST_O_SEARCH).map_err(|err| err as i32);
 
         let res = res.and_then(|fd| {
-            if unsafe { libc::fchdir(fd.as_raw_fd()) } == 0 {
-                Ok(fd)
-            } else {
-                Err(errno::errno().0)
+            match fchdir(&fd) {
+                Ok(()) => Ok(fd),
+                // nix::Result::Err contains nix::errno::Errno, which does not offer an API for
+                // converting to a raw int.
+                Err(_) => Err(errno::errno().0),
             }
         });
 
@@ -139,35 +144,33 @@ pub fn cd(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
     if best_errno == ENOTDIR {
         streams
             .err
-            .append(&wgettext_fmt!("%s: '%s' is not a directory\n", cmd, dir_in));
+            .appendln(&wgettext_fmt!("%s: '%s' is not a directory", cmd, dir_in));
     } else if !broken_symlink.is_empty() {
-        streams.err.append(&wgettext_fmt!(
-            "%s: '%s' is a broken symbolic link to '%s'\n",
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: '%s' is a broken symbolic link to '%s'",
             cmd,
             broken_symlink,
             broken_symlink_target
         ));
     } else if best_errno == ELOOP {
-        streams.err.append(&wgettext_fmt!(
-            "%s: Too many levels of symbolic links: '%s'\n",
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: Too many levels of symbolic links: '%s'",
             cmd,
             dir_in
         ));
     } else if best_errno == ENOENT {
-        streams.err.append(&wgettext_fmt!(
-            "%s: The directory '%s' does not exist\n",
-            cmd,
-            dir_in
-        ));
+        streams
+            .err
+            .appendln(&wgettext_fmt!(DIR_DOES_NOT_EXIST, cmd, dir_in));
     } else if best_errno == EACCES || best_errno == EPERM {
         streams
             .err
-            .append(&wgettext_fmt!("%s: Permission denied: '%s'\n", cmd, dir_in));
+            .appendln(&wgettext_fmt!("%s: Permission denied: '%s'", cmd, dir_in));
     } else {
         errno::set_errno(Errno(best_errno));
         wperror(L!("cd"));
-        streams.err.append(&wgettext_fmt!(
-            "%s: Unknown error trying to locate directory '%s'\n",
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: Unknown error trying to locate directory '%s'",
             cmd,
             dir_in
         ));
