@@ -5,8 +5,8 @@ use crate::io::IoStreams;
 use crate::parser::Parser;
 use crate::proc::{Job, add_disowned_job};
 use crate::{builtins::shared::HelpOnlyCmdOpts, localization::wgettext_fmt};
-use fish_wchar::wstr;
-use libc::SIGCONT;
+use fish_widestring::wstr;
+use nix::sys::signal::{Signal, killpg};
 
 /// Helper for builtin_disown.
 fn disown_job(cmd: &wstr, streams: &mut IoStreams, j: &Job) {
@@ -19,12 +19,10 @@ fn disown_job(cmd: &wstr, streams: &mut IoStreams, j: &Job) {
     let pgid = j.get_pgid();
     if j.is_stopped() {
         if let Some(pgid) = pgid {
-            unsafe {
-                libc::killpg(pgid.as_pid_t(), SIGCONT);
-            }
+            let _ = killpg(pgid.as_nix_pid(), Some(Signal::SIGCONT));
         }
-        streams.err.append(&wgettext_fmt!(
-            "%s: job %d ('%s') was stopped and has been signalled to continue.\n",
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: job %d ('%s') was stopped and has been signalled to continue.",
             cmd,
             j.job_id(),
             j.command()
@@ -68,7 +66,7 @@ pub fn disown(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         } else {
             streams
                 .err
-                .append(&wgettext_fmt!("%s: There are no suitable jobs\n", cmd));
+                .appendln(&wgettext_fmt!(BUILTIN_ERR_NO_SUITABLE_JOBS, cmd));
             retval = Err(STATUS_CMD_ERROR);
         }
     } else {
@@ -91,7 +89,7 @@ pub fn disown(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                     // Valid identifier but no such job
                     streams
                         .err
-                        .append(&wgettext_fmt!("%s: Could not find job '%d'\n", cmd, pid));
+                        .appendln(&wgettext_fmt!(BUILTIN_ERR_COULD_NOT_FIND_JOB, cmd, pid));
                     None
                 })
             })
