@@ -6,10 +6,10 @@ use crate::io::IoStreams;
 use crate::job_group::{JobId, MaybeJobId};
 use crate::localization::{wgettext, wgettext_fmt};
 use crate::parser::Parser;
-use crate::proc::{Job, clock_ticks_to_seconds, have_proc_stat, proc_get_jiffies};
-use crate::wgetopt::{ArgType, WGetopter, WOption, wopt};
+use crate::proc::{HAVE_PROC_STAT, Job, clock_ticks_to_seconds, proc_get_jiffies};
 use crate::wutil::fish_wcstoi;
-use fish_wchar::{L, WExt, WString, wstr};
+use fish_wgetopt::{ArgType, WGetopter, WOption, wopt};
+use fish_widestring::{L, WExt as _, WString, wstr};
 use std::num::NonZeroU32;
 
 /// Print modes for the jobs builtin.
@@ -43,7 +43,7 @@ fn cpu_use(j: &Job) -> f64 {
 fn builtin_jobs_print(j: &Job, mode: JobsPrintMode, header: bool, streams: &mut IoStreams) {
     let pgid = match j.get_pgid() {
         Some(pgid) => pgid.to_string(),
-        None => "-".to_string(),
+        None => "-".to_owned(),
     };
 
     let mut out = WString::new();
@@ -52,16 +52,23 @@ fn builtin_jobs_print(j: &Job, mode: JobsPrintMode, header: bool, streams: &mut
         JobsPrintMode::Default => {
             if header {
                 // Print table header before first job.
-                out += wgettext!("Job\tGroup\t");
-                if have_proc_stat() {
-                    out += wgettext!("CPU\t");
+                out += wgettext!("Job");
+                out.push('\t');
+                out += wgettext!("Group");
+                out.push('\t');
+                if *HAVE_PROC_STAT {
+                    out += wgettext!("CPU");
+                    out.push('\t');
                 }
-                out += wgettext!("State\tCommand\n");
+                out += wgettext!("State");
+                out.push('\t');
+                out += wgettext!("Command");
+                out.push('\n');
             }
 
             sprintf!(=> &mut out, "%d\t%s\t", j.job_id(), pgid);
 
-            if have_proc_stat() {
+            if *HAVE_PROC_STAT {
                 sprintf!(=> &mut out, "%.0f%%\t", 100.0 * cpu_use(j));
             }
 
@@ -84,7 +91,8 @@ fn builtin_jobs_print(j: &Job, mode: JobsPrintMode, header: bool, streams: &mut
         JobsPrintMode::PrintGroup => {
             if header {
                 // Print table header before first job.
-                out += wgettext!("Group\n");
+                out += wgettext!("Group");
+                out.push('\n');
             }
             out += &sprintf!("%s\n", pgid)[..];
             streams.out.append(&out);
@@ -92,7 +100,8 @@ fn builtin_jobs_print(j: &Job, mode: JobsPrintMode, header: bool, streams: &mut
         JobsPrintMode::PrintPid => {
             if header {
                 // Print table header before first job.
-                out += wgettext!("Process\n");
+                out += wgettext!("Process");
+                out.push('\n');
             }
 
             for p in j.external_procs() {
@@ -103,7 +112,8 @@ fn builtin_jobs_print(j: &Job, mode: JobsPrintMode, header: bool, streams: &mut
         JobsPrintMode::PrintCommand => {
             if header {
                 // Print table header before first job.
-                out += wgettext!("Command\n");
+                out += wgettext!("Command");
+                out.push('\n');
             }
 
             for p in j.processes() {
@@ -111,7 +121,7 @@ fn builtin_jobs_print(j: &Job, mode: JobsPrintMode, header: bool, streams: &mut
             }
             streams.out.append(&out);
         }
-    };
+    }
 }
 
 const SHORT_OPTIONS: &wstr = L!("cghlpq");
@@ -192,8 +202,8 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
             if arg.char_at(0) == '%' {
                 match fish_wcstoi(&arg[1..]).ok().filter(|&job_id| job_id >= 0) {
                     None => {
-                        streams.err.append(&wgettext_fmt!(
-                            "%s: '%s' is not a valid job ID\n",
+                        streams.err.appendln(&wgettext_fmt!(
+                            "%s: '%s' is not a valid job ID",
                             cmd,
                             arg
                         ));
@@ -212,7 +222,7 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                 }
             } else {
                 let pid = parse_pid(streams, cmd, arg)?;
-                j = parser.job_get_from_pid(pid)
+                j = parser.job_get_from_pid(pid);
             }
 
             if let Some(j) = j.filter(|j| !j.is_completed() && j.is_constructed()) {
@@ -222,7 +232,7 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                 if mode != JobsPrintMode::PrintNothing {
                     streams
                         .err
-                        .append(&wgettext_fmt!("%s: No suitable job: %s\n", cmd, arg));
+                        .appendln(&wgettext_fmt!("%s: No suitable job: %s", cmd, arg));
                 }
                 return Err(STATUS_CMD_ERROR);
             }
@@ -242,7 +252,7 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
         if !streams.out_is_redirected && mode != JobsPrintMode::PrintNothing {
             streams
                 .out
-                .append(&wgettext_fmt!("%s: There are no jobs\n", argv[0]));
+                .appendln(&wgettext_fmt!("%s: There are no jobs", argv[0]));
         }
         return Err(STATUS_CMD_ERROR);
     }
