@@ -1,19 +1,19 @@
-use crate::env::environment::Environment;
+use crate::env::environment::Environment as _;
 use std::fs::Metadata;
-use std::os::unix::prelude::{FileTypeExt, MetadataExt};
+use std::os::unix::prelude::{FileTypeExt as _, MetadataExt as _};
 use std::time::SystemTime;
 
 use super::prelude::*;
-use crate::nix::{getegid, geteuid};
 use crate::path::path_apply_working_directory;
-use crate::util::wcsfilecmp_glob;
-use crate::wcstringutil::split_string_tok;
 use crate::wutil::{
     INVALID_FILE_ID, file_id_for_path, lwstat, normalize_path, waccess, wbasename, wdirname,
     wrealpath, wstat,
 };
 use bitflags::bitflags;
-use libc::{F_OK, PATH_MAX, R_OK, S_ISGID, S_ISUID, W_OK, X_OK, mode_t};
+use fish_util::wcsfilecmp_glob;
+use fish_wcstringutil::split_string_tok;
+use libc::{PATH_MAX, S_ISGID, S_ISUID, mode_t};
+use nix::unistd::{AccessFlags, Gid, Uid};
 
 macro_rules! path_error {
     (
@@ -23,7 +23,7 @@ macro_rules! path_error {
     $(,)?
     ) => {
         $streams.err.append(L!("path "));
-        $streams.err.append(&wgettext_fmt!($string, $($args),*));
+        $streams.err.appendln(&wgettext_fmt!($string, $($args),*));
     };
 }
 
@@ -279,11 +279,11 @@ fn parse_opts<'args>(
                 continue;
             }
             't' if opts.types_valid => {
-                let types = opts.types.get_or_insert_with(TypeFlags::default);
+                let types = opts.types.get_or_insert_default();
                 let types_args = split_string_tok(w.woptarg.unwrap(), L!(","), None);
                 for t in types_args {
                     let Ok(r#type) = t.try_into() else {
-                        path_error!(streams, "%s: Invalid type '%s'\n", "path", t);
+                        path_error!(streams, "%s: Invalid type '%s'", "path", t);
                         return Err(STATUS_INVALID_ARGS);
                     };
                     *types |= r#type;
@@ -291,11 +291,11 @@ fn parse_opts<'args>(
                 continue;
             }
             'p' if opts.perms_valid => {
-                let perms = opts.perms.get_or_insert_with(PermFlags::default);
+                let perms = opts.perms.get_or_insert_default();
                 let perms_args = split_string_tok(w.woptarg.unwrap(), L!(","), None);
                 for p in perms_args {
                     let Ok(perm) = p.try_into() else {
-                        path_error!(streams, "%s: Invalid permission '%s'\n", "path", p);
+                        path_error!(streams, "%s: Invalid permission '%s'", "path", p);
                         return Err(STATUS_INVALID_ARGS);
                     };
                     *perms |= perm;
@@ -307,32 +307,32 @@ fn parse_opts<'args>(
                 continue;
             }
             'r' if opts.perms_valid => {
-                let perms = opts.perms.get_or_insert_with(PermFlags::default);
+                let perms = opts.perms.get_or_insert_default();
                 *perms |= PermFlags::READ;
                 continue;
             }
             'w' if opts.perms_valid => {
-                let perms = opts.perms.get_or_insert_with(PermFlags::default);
+                let perms = opts.perms.get_or_insert_default();
                 *perms |= PermFlags::WRITE;
                 continue;
             }
             'x' if opts.perms_valid => {
-                let perms = opts.perms.get_or_insert_with(PermFlags::default);
+                let perms = opts.perms.get_or_insert_default();
                 *perms |= PermFlags::EXEC;
                 continue;
             }
             'f' if opts.types_valid => {
-                let types = opts.types.get_or_insert_with(TypeFlags::default);
+                let types = opts.types.get_or_insert_default();
                 *types |= TypeFlags::FILE;
                 continue;
             }
             'l' if opts.types_valid => {
-                let types = opts.types.get_or_insert_with(TypeFlags::default);
+                let types = opts.types.get_or_insert_default();
                 *types |= TypeFlags::LINK;
                 continue;
             }
             'd' if opts.types_valid => {
-                let types = opts.types.get_or_insert_with(TypeFlags::default);
+                let types = opts.types.get_or_insert_default();
                 *types |= TypeFlags::DIR;
                 continue;
             }
@@ -368,7 +368,7 @@ fn parse_opts<'args>(
     *optind = w.wopt_index;
 
     if n_req_args != 0 {
-        assert!(n_req_args == 1);
+        assert_eq!(n_req_args, 1);
         opts.arg1 = args.get(*optind).copied();
         if opts.arg1.is_some() {
             *optind += 1;
@@ -424,7 +424,7 @@ fn path_transform(
             // TODO: Is that correct?
             if opts.quiet {
                 return Ok(SUCCESS);
-            };
+            }
         }
         path_out(streams, &opts, transformed);
     }
@@ -698,7 +698,7 @@ fn path_sort(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bu
         }
         None => wbasename,
         Some(k) => {
-            path_error!(streams, "%s: Invalid sort key '%s'\n", args[0], k);
+            path_error!(streams, "%s: Invalid sort key '%s'", args[0], k);
             return Err(STATUS_INVALID_ARGS);
         }
     };
@@ -743,7 +743,7 @@ fn path_sort(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bu
     Ok(SUCCESS)
 }
 
-fn filter_path(opts: &Options, path: &wstr, uid: Option<u32>, gid: Option<u32>) -> bool {
+fn filter_path(opts: &Options, path: &wstr, uid: Option<Uid>, gid: Option<Gid>) -> bool {
     // TODO: Add moar stuff:
     // fifos, sockets, size greater than zero, setuid, ...
     // Nothing to check, file existence is checked elsewhere.
@@ -758,7 +758,7 @@ fn filter_path(opts: &Options, path: &wstr, uid: Option<u32>, gid: Option<u32>)
         let mut type_ok = false;
         if t.contains(TypeFlags::LINK) {
             let md = lwstat(path);
-            type_ok = md.map(|x| x.is_symlink()).unwrap_or(false);
+            type_ok = md.as_ref().is_ok_and(Metadata::is_symlink);
         }
         let Ok(md) = wstat(path) else {
             // Does not exist
@@ -784,7 +784,7 @@ fn filter_path(opts: &Options, path: &wstr, uid: Option<u32>, gid: Option<u32>)
     }
 
     if let Some(perm) = opts.perms {
-        let mut amode = 0;
+        let mut amode = AccessFlags::empty();
         // TODO: Update bitflags so this works
         /*
         for f in perm {
@@ -797,21 +797,19 @@ fn filter_path(opts: &Options, path: &wstr, uid: Option<u32>, gid: Option<u32>)
         }
         */
         if perm.contains(PermFlags::READ) {
-            amode |= R_OK;
+            amode.insert(AccessFlags::R_OK);
         }
         if perm.contains(PermFlags::WRITE) {
-            amode |= W_OK;
+            amode.insert(AccessFlags::W_OK);
         }
         if perm.contains(PermFlags::EXEC) {
-            amode |= X_OK;
+            amode.insert(AccessFlags::X_OK);
         }
-        // access returns 0 on success,
-        // -1 on failure. Yes, C can't even keep its bools straight.
         // Skip this if we don't have a mode to check - the stat can do existence too.
         // It's tempting to check metadata here if we have it,
         // e.g. see if any read-bit is set for READ.
         // That won't work for root.
-        if amode != 0 && waccess(path, amode) != 0 {
+        if !amode.is_empty() && waccess(path, amode).is_err() {
             return false;
         }
 
@@ -832,9 +830,9 @@ fn filter_path(opts: &Options, path: &wstr, uid: Option<u32>, gid: Option<u32>)
                 return false;
             } else if perm.contains(PermFlags::SGID) && (md.mode() as mode_t & S_ISGID) == 0 {
                 return false;
-            } else if perm.contains(PermFlags::USER) && uid != Some(md.uid()) {
+            } else if perm.contains(PermFlags::USER) && uid.map(|u| u.as_raw()) != Some(md.uid()) {
                 return false;
-            } else if perm.contains(PermFlags::GROUP) && gid != Some(md.gid()) {
+            } else if perm.contains(PermFlags::GROUP) && gid.map(|g| g.as_raw()) != Some(md.gid()) {
                 return false;
             }
         }
@@ -874,12 +872,12 @@ fn path_filter_maybe_is(
 
     // If we're looking for the owner/group, get our euid/egid here once.
     let uid = if opts.perms.unwrap_or_default().contains(PermFlags::USER) {
-        Some(geteuid())
+        Some(Uid::effective())
     } else {
         None
     };
     let gid = if opts.perms.unwrap_or_default().contains(PermFlags::GROUP) {
-        Some(getegid())
+        Some(Gid::effective())
     } else {
         None
     };
@@ -893,7 +891,7 @@ fn path_filter_maybe_is(
     }) {
         // If we don't have filters, check if it exists.
         if opts.perms.is_none() && opts.types.is_none() {
-            let ok = waccess(arg, F_OK) == 0;
+            let ok = waccess(arg, AccessFlags::F_OK).is_ok();
             if ok == opts.invert {
                 // For --all, fail early if any path does not match the filter.
                 if opts.all {
@@ -921,7 +919,7 @@ fn path_filter_maybe_is(
         }
         if opts.quiet {
             return Ok(SUCCESS);
-        };
+        }
     }
 
     if opts.all && n_transformed != arguments_vec.len() {
@@ -953,7 +951,7 @@ pub fn path(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bui
     if argc <= 1 {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_MISSING_SUBCMD, cmd));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_MISSING_SUBCMD, cmd));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
@@ -979,7 +977,7 @@ pub fn path(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bui
         _ => {
             streams
                 .err
-                .append(&wgettext_fmt!(BUILTIN_ERR_INVALID_SUBCMD, cmd, subcmd_name));
+                .appendln(&wgettext_fmt!(BUILTIN_ERR_INVALID_SUBCMD, cmd, subcmd_name));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
