@@ -3,7 +3,7 @@
 use errno::errno;
 
 use super::prelude::*;
-use crate::env::Environment;
+use crate::env::Environment as _;
 use crate::{
     path::path_apply_working_directory,
     wutil::{normalize_path, wrealpath},
