@@ -21,14 +21,7 @@ use crate::{
     env::{EnvMode, EnvVar, Environment},
     wutil::wcstoi::wcstoi_partial,
 };
-
-localizable_consts!(
-    MISMATCHED_ARGS
-    "%s: given %d indexes but %d values\n"
-
-    UVAR_ERR
-    "%s: successfully set universal '%s'; but a global by that name shadows it\n"
-);
+use fish_common::help_section;
 
 #[derive(Debug, Clone)]
 struct Options {
@@ -185,7 +178,7 @@ impl Options {
                     // implicit drop(w); here
                     if args[optind - 1].starts_with("-o") {
                         // TODO: translate this
-                        streams.err.appendln(sprintf!(
+                        streams.err.appendln(&sprintf!(
                             "Fish does not have shell options. See `help %s`.",
                             help_section!("fish_for_bash_users")
                         ));
@@ -233,14 +226,14 @@ impl Options {
     ) -> Result<(), ErrorCode> {
         // Can't query and erase or list.
         if opts.query && (opts.erase || opts.list) {
-            streams.err.append(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
+            streams.err.appendln(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
 
         // We can't both list and erase variables.
         if opts.erase && opts.list {
-            streams.err.append(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
+            streams.err.appendln(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -254,7 +247,9 @@ impl Options {
             // ..unless we are erasing a variable, in which case we can erase from several in one go.
             && !opts.erase
         {
-            streams.err.append(&wgettext_fmt!(BUILTIN_ERR_GLOCAL, cmd));
+            streams
+                .err
+                .appendln(&wgettext_fmt!(BUILTIN_ERR_GLOCAL, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -263,7 +258,7 @@ impl Options {
         if opts.exportv && opts.unexport {
             streams
                 .err
-                .append(&wgettext_fmt!(BUILTIN_ERR_EXPUNEXP, cmd));
+                .appendln(&wgettext_fmt!(BUILTIN_ERR_EXPUNEXP, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -272,14 +267,14 @@ impl Options {
         if opts.pathvar && opts.unpathvar {
             streams
                 .err
-                .append(&wgettext_fmt!(BUILTIN_ERR_PATHUNPATH, cmd));
+                .appendln(&wgettext_fmt!(BUILTIN_ERR_PATHUNPATH, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
 
         // Trying to erase and (un)export at the same time doesn't make sense.
         if opts.erase && (opts.exportv || opts.unexport) {
-            streams.err.append(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
+            streams.err.appendln(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -294,7 +289,7 @@ impl Options {
                 || opts.exportv
                 || opts.universal)
         {
-            streams.err.append(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
+            streams.err.appendln(&wgettext_fmt!(BUILTIN_ERR_COMBO, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -302,7 +297,7 @@ impl Options {
         if args.len() == optind && opts.erase {
             streams
                 .err
-                .append(&wgettext_fmt!(BUILTIN_ERR_MISSING, cmd, L!("--erase")));
+                .appendln(&wgettext_fmt!(BUILTIN_ERR_MISSING, cmd, L!("--erase")));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -324,7 +319,11 @@ fn warn_if_uvar_shadows_global(
         && parser.is_interactive()
         && parser.vars().getf(dest, EnvMode::GLOBAL).is_some()
     {
-        streams.err.append(&wgettext_fmt!(UVAR_ERR, cmd, dest));
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: successfully set universal '%s'; but a global by that name shadows it",
+            cmd,
+            dest
+        ));
     }
 }
 
@@ -332,29 +331,29 @@ fn handle_env_return(retval: EnvStackSetResult, cmd: &wstr, key: &wstr, streams:
     match retval {
         EnvStackSetResult::Ok => (),
         EnvStackSetResult::Perm => {
-            streams.err.append(&wgettext_fmt!(
-                "%s: Tried to change the read-only variable '%s'\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: Tried to change the read-only variable '%s'",
                 cmd,
                 key
             ));
         }
         EnvStackSetResult::Scope => {
-            streams.err.append(&wgettext_fmt!(
-                "%s: Tried to modify the special variable '%s' with the wrong scope\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: Tried to modify the special variable '%s' with the wrong scope",
                 cmd,
                 key
             ));
         }
         EnvStackSetResult::Invalid => {
-            streams.err.append(&wgettext_fmt!(
-                "%s: Tried to modify the special variable '%s' to an invalid value\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: Tried to modify the special variable '%s' to an invalid value",
                 cmd,
                 key
             ));
         }
         EnvStackSetResult::NotFound => {
-            streams.err.append(&wgettext_fmt!(
-                "%s: The variable '%s' does not exist\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: The variable '%s' does not exist",
                 cmd,
                 key
             ));
@@ -393,11 +392,10 @@ impl std::fmt::Display for EnvArrayParseError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         writeln!(
             f,
-            "{}",
+            "{}\n",
             match self {
                 EnvArrayParseError::InvalidIndex(varname) =>
-                    wgettext_fmt!("%s: Invalid index starting at '%s'\n", "set", varname)
-                        .to_string(),
+                    wgettext_fmt!("%s: Invalid index starting at '%s'", "set", varname).to_string(),
             }
         )
     }
@@ -433,12 +431,8 @@ fn split_var_and_indexes<'a>(
 ) -> Option<SplitVar<'a>> {
     match split_var_and_indexes_internal(arg, mode, vars) {
         Ok(split) => Some(split),
-        Err(EnvArrayParseError::InvalidIndex(varname)) => {
-            streams.err.append(&wgettext_fmt!(
-                "%s: Invalid index starting at '%s'\n",
-                "set",
-                &varname,
-            ));
+        Err(env_array_parse_error) => {
+            streams.err.append(&format!("{env_array_parse_error}"));
             None
         }
     }
@@ -573,7 +567,7 @@ fn list(opts: &Options, parser: &Parser, streams: &mut IoStreams) -> BuiltinResu
                     if i > 1 {
                         val.push(' ');
                     }
-                    val += &expand_escape_string(history.item_at_index(i).unwrap().str())[..]
+                    val += &expand_escape_string(history.item_at_index(i).unwrap().str())[..];
                 }
             } else if let Some(var) = parser.vars().getf_unless_empty(&key, opts.env_mode()) {
                 val = expand_escape_variable(&var);
@@ -658,9 +652,11 @@ fn show_scope(var_name: &wstr, scope: EnvMode, streams: &mut IoStreams, vars: &d
         wgettext!("unexported")
     };
     let pathvarv = if var.is_pathvar() {
-        wgettext!(" a path variable")
+        let mut pathvarv = L!(" ").to_owned();
+        pathvarv.push_utfstr(wgettext!("a path variable"));
+        pathvarv
     } else {
-        L!("")
+        L!("").to_owned()
     };
     let vals = var.as_list();
     streams.out.append(&wgettext_fmt!(
@@ -674,21 +670,24 @@ fn show_scope(var_name: &wstr, scope: EnvMode, streams: &mut IoStreams, vars: &d
     // HACK: PWD can be set, depending on how you ask.
     // For our purposes it's read-only.
     if EnvVar::flags_for(var_name).contains(EnvVarFlags::READ_ONLY) {
-        streams.out.append(wgettext!(" (read-only)\n"));
+        streams
+            .out
+            .appendln(" ".chars().chain(wgettext!("(read-only)").chars()));
     } else {
-        streams.out.append_char('\n');
+        streams.out.append('\n');
     }
 
     for i in 0..vals.len() {
         if vals.len() > 100 {
             if i == 50 {
                 // try to print a mid-line ellipsis because we are eliding lines not words
-                streams.out.append(if u32::from(get_ellipsis_char()) > 256 {
-                    L!("\u{22EF}")
-                } else {
-                    get_ellipsis_str()
-                });
-                streams.out.append_char('\n');
+                streams
+                    .out
+                    .appendln(if u32::from(get_ellipsis_char()) > 256 {
+                        L!("\u{22EF}")
+                    } else {
+                        get_ellipsis_str()
+                    });
             }
             if i >= 50 && i < vals.len() - 50 {
                 continue;
@@ -707,6 +706,10 @@ fn show_scope(var_name: &wstr, scope: EnvMode, streams: &mut IoStreams, vars: &d
 
 /// Show mode. Show information about the named variable(s).
 fn show(cmd: &wstr, parser: &Parser, streams: &mut IoStreams, args: &[&wstr]) -> BuiltinResult {
+    localizable_consts! {
+        ORIGINALLY_INHERITED_AS
+        "$%s: originally inherited as |%s|"
+    }
     let vars = parser.vars();
     if args.is_empty() {
         // show all vars
@@ -726,11 +729,9 @@ fn show(cmd: &wstr, parser: &Parser, streams: &mut IoStreams, args: &[&wstr]) ->
                     inherited,
                     EscapeStringStyle::Script(EscapeFlags::NO_PRINTABLES | EscapeFlags::NO_QUOTED),
                 );
-                streams.out.append(&wgettext_fmt!(
-                    "$%s: originally inherited as |%s|\n",
-                    name,
-                    escaped_val
-                ));
+                streams
+                    .out
+                    .appendln(&wgettext_fmt!(ORIGINALLY_INHERITED_AS, name, escaped_val));
             }
         }
     } else {
@@ -742,8 +743,8 @@ fn show(cmd: &wstr, parser: &Parser, streams: &mut IoStreams, args: &[&wstr]) ->
             }
 
             if arg.contains('[') {
-                streams.err.append(&wgettext_fmt!(
-                    "%s: `set --show` does not allow slices with the var names\n",
+                streams.err.appendln(&wgettext_fmt!(
+                    "%s: `set --show` does not allow slices with the var names",
                     cmd
                 ));
                 builtin_print_error_trailer(parser, streams.err, cmd);
@@ -758,11 +759,9 @@ fn show(cmd: &wstr, parser: &Parser, streams: &mut IoStreams, args: &[&wstr]) ->
                     inherited,
                     EscapeStringStyle::Script(EscapeFlags::NO_PRINTABLES | EscapeFlags::NO_QUOTED),
                 );
-                streams.out.append(&wgettext_fmt!(
-                    "$%s: originally inherited as |%s|\n",
-                    arg,
-                    escaped_val
-                ));
+                streams
+                    .out
+                    .appendln(&wgettext_fmt!(ORIGINALLY_INHERITED_AS, arg, escaped_val));
             }
         }
     }
@@ -890,7 +889,7 @@ fn new_var_values(
         // So do not use the given variable: we must re-fetch it.
         // TODO: this races under concurrent execution.
         if let Some(existing) = vars.get(varname) {
-            result = existing.as_list().to_owned();
+            existing.as_list().clone_into(&mut result);
         }
 
         if opts.prepend {
@@ -906,18 +905,21 @@ fn new_var_values(
 
 /// This handles the more difficult case of setting individual slices of a var.
 fn new_var_values_by_index(split: &SplitVar, argv: &[&wstr]) -> Vec<WString> {
-    assert!(
-        argv.len() == split.indexes.len(),
+    assert_eq!(
+        argv.len(),
+        split.indexes.len(),
         "Must have the same number of indexes as arguments"
     );
 
     // Inherit any existing values.
     // Note unlike the append/prepend case, we start with a variable in the same scope as we are
     // setting.
-    let mut result = vec![];
-    if let Some(var) = split.var.as_ref() {
-        result = var.as_list().to_owned();
-    }
+    let mut result = split
+        .var
+        .as_ref()
+        .map(EnvVar::as_list)
+        .unwrap_or_default()
+        .to_owned();
 
     // For each (index, argument) pair, set the element in our `result` to the replacement string.
     // Extend the list with empty strings as needed. The indexes are 1-based.
@@ -946,7 +948,7 @@ fn set_internal(
     if argv.is_empty() {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_MIN_ARG_COUNT1, cmd, 1));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_MIN_ARG_COUNT1, cmd, 1));
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
     }
@@ -982,7 +984,7 @@ fn set_internal(
             if *ind <= 0 {
                 streams
                     .err
-                    .append(&wgettext_fmt!("%s: array index out of bounds\n", cmd));
+                    .appendln(&wgettext_fmt!("%s: array index out of bounds", cmd));
                 builtin_print_error_trailer(parser, streams.err, cmd);
                 return Err(STATUS_INVALID_ARGS);
             }
@@ -999,8 +1001,8 @@ fn set_internal(
 
         // Argument count and index count must agree.
         if split.indexes.len() != argv.len() {
-            streams.err.append(&wgettext_fmt!(
-                MISMATCHED_ARGS,
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: given %d indexes but %d values",
                 cmd,
                 split.indexes.len(),
                 argv.len()
