@@ -1,15 +1,16 @@
 use super::prelude::*;
-use crate::common::{Named, bytes2wcstring, escape, get_by_sorted_name};
+use crate::common::{Named, bytes2wcstring, escape, get_by_sorted_name, str2wcstring};
 use crate::fds::BorrowedFdFile;
 use crate::io::OutputStream;
 use crate::parse_constants::UNKNOWN_BUILTIN_ERR_MSG;
-use crate::parse_util::parse_util_argument_is_help;
+use crate::parse_util::argument_is_help;
 use crate::parser::{BlockType, LoopStatus};
 use crate::proc::{Pid, ProcStatus, no_exec};
 use crate::{builtins::*, wutil};
 use errno::errno;
-use fish_wchar::L;
-use std::io::{BufRead, BufReader, Read};
+use fish_common::assert_sorted_by_name;
+use fish_widestring::L;
+use std::io::{BufRead as _, BufReader, Read as _};
 
 pub type BuiltinCmd = fn(&Parser, &mut IoStreams, &mut [&wstr]) -> BuiltinResult;
 
@@ -20,87 +21,108 @@ pub const DEFAULT_READ_PROMPT: &wstr =
 localizable_consts!(
     /// Error message on missing argument.
     pub BUILTIN_ERR_MISSING
-    "%s: %s: option requires an argument\n"
+    "%s: %s: option requires an argument"
 
     /// Error message on unexpected argument.
     pub BUILTIN_ERR_UNEXP_ARG
-    "%s: %s: option does not take an argument\n"
+    "%s: %s: option does not take an argument"
 
     /// Error message on missing man page.
     pub BUILTIN_ERR_MISSING_HELP
-    "fish: %s: missing man page\nDocumentation may not be installed.\n`help %s` will show an online version\n"
+    "fish: %s: missing man page\nDocumentation may not be installed.\n`help %s` will show an online version"
 
     /// Error message on multiple scope levels for variables.
     pub BUILTIN_ERR_GLOCAL
-    "%s: scope can be only one of: universal function global local\n"
+    "%s: scope can be only one of: universal function global local"
 
     /// Error message for specifying both export and unexport to set/read.
     pub BUILTIN_ERR_EXPUNEXP
-    "%s: cannot both export and unexport\n"
+    "%s: cannot both export and unexport"
 
     /// Error message for specifying both path and unpath to set/read.
     pub BUILTIN_ERR_PATHUNPATH
-    "%s: cannot both path and unpath\n"
+    "%s: cannot both path and unpath"
 
     /// Error message for unknown switch.
     pub BUILTIN_ERR_UNKNOWN
-    "%s: %s: unknown option\n"
+    "%s: %s: unknown option"
 
     /// Error message for invalid bind mode name.
     pub BUILTIN_ERR_BIND_MODE
-    "%s: %s: invalid mode name. See `help %s`\n"
+    "%s: %s: invalid mode name. See `help %s`"
 
     /// Error message when too many arguments are supplied to a builtin.
     pub BUILTIN_ERR_TOO_MANY_ARGUMENTS
-    "%s: too many arguments\n"
+    "%s: too many arguments"
 
     /// Error message when integer expected
     pub BUILTIN_ERR_NOT_NUMBER
-    "%s: %s: invalid integer\n"
+    "%s: %s: invalid integer"
 
     /// Command that requires a subcommand was invoked without a recognized subcommand.
     pub BUILTIN_ERR_MISSING_SUBCMD
-    "%s: missing subcommand\n"
+    "%s: missing subcommand"
 
     pub BUILTIN_ERR_INVALID_SUBCMD
-    "%s: %s: invalid subcommand\n"
+    "%s: %s: invalid subcommand"
 
     pub BUILTIN_ERR_INVALID_SUBSUBCMD
-    "%s %s: %s: invalid subcommand\n"
+    "%s %s: %s: invalid subcommand"
 
     /// Error messages for unexpected args.
     pub BUILTIN_ERR_ARG_COUNT0
-    "%s: missing argument\n"
+    "%s: missing argument"
 
     pub BUILTIN_ERR_ARG_COUNT1
-    "%s: expected %d arguments; got %d\n"
+    "%s: expected %d arguments; got %d"
 
     pub BUILTIN_ERR_ARG_COUNT2
-    "%s: %s: expected %d arguments; got %d\n"
+    "%s: %s: expected %d arguments; got %d"
 
     pub BUILTIN_ERR_MIN_ARG_COUNT1
-    "%s: expected >= %d arguments; got %d\n"
+    "%s: expected >= %d arguments; got %d"
 
     pub BUILTIN_ERR_MAX_ARG_COUNT1
-    "%s: expected <= %d arguments; got %d\n"
+    "%s: expected <= %d arguments; got %d"
 
     /// Error message for invalid variable name.
     pub BUILTIN_ERR_VARNAME
-    "%s: %s: invalid variable name. See `help %s`\n"
+    "%s: %s: invalid variable name. See `help %s`"
 
     /// Error message on invalid combination of options.
     pub BUILTIN_ERR_COMBO
-    "%s: invalid option combination\n"
+    "%s: invalid option combination"
 
     pub BUILTIN_ERR_COMBO2
-    "%s: invalid option combination, %s\n"
+    "%s: invalid option combination, %s"
 
     pub BUILTIN_ERR_COMBO2_EXCLUSIVE
-    "%s: %s %s: options cannot be used together\n"
+    "%s: %s %s: options cannot be used together"
+
+    pub BUILTIN_ERR_REGEX_COMPILE
+    "%s: Regular expression compile error: %s"
+
+    pub BUILTIN_ERR_NO_SUITABLE_JOBS
+    "%s: There are no suitable jobs"
+
+    pub BUILTIN_ERR_COULD_NOT_FIND_JOB
+    "%s: Could not find job '%d'"
+
+    pub BUILTIN_ERR_STDIN_CLOSED
+    "%s: stdin is closed"
+
+    pub BUILTIN_ERR_INVALID_MAX_MATCHES
+    "%s: Invalid max matches value '%s'"
+
+    pub BUILTIN_ERR_INVALID_MAX_VALUE
+    "%s: Invalid max value '%s'"
 
     /// The send stuff to foreground message.
     pub FG_MSG
-    "Send job %d (%s) to foreground\n"
+    "Send job %d (%s) to foreground"
+
+    pub VERSION_STRING_TEMPLATE
+    "%s, version %s"
 );
 
 // Return values (`$status` values for fish scripts) for various situations.
@@ -475,7 +497,7 @@ pub fn builtin_run(parser: &Parser, argv: &mut [&wstr], streams: &mut IoStreams)
     // We can be handed a keyword by the parser as if it was a command. This happens when the user
     // follows the keyword by `-h` or `--help`. Since it isn't really a builtin command we need to
     // handle displaying help for it here.
-    if argv.len() == 2 && parse_util_argument_is_help(argv[1]) && cmd_needs_help(argv[0]) {
+    if argv.len() == 2 && argument_is_help(argv[1]) && cmd_needs_help(argv[0]) {
         builtin_print_help(parser, streams, argv[0]);
         return ProcStatus::from_exit_code(STATUS_CMD_OK);
     }
@@ -631,7 +653,7 @@ pub fn builtin_print_help(parser: &Parser, streams: &mut IoStreams, cmd: &wstr)
     if res.status.normal_exited() && res.status.exit_code() == 2 {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_MISSING_HELP, name_esc, name_esc));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_MISSING_HELP, name_esc, name_esc));
     }
 }
 
@@ -645,7 +667,7 @@ pub fn builtin_unknown_option(
 ) {
     streams
         .err
-        .append(&wgettext_fmt!(BUILTIN_ERR_UNKNOWN, cmd, opt));
+        .appendln(&wgettext_fmt!(BUILTIN_ERR_UNKNOWN, cmd, opt));
     if print_hints {
         builtin_print_error_trailer(parser, streams.err, cmd);
     }
@@ -662,7 +684,7 @@ pub fn builtin_missing_argument(
     if opt.char_at(0) == '-' && opt.char_at(1) != '-' {
         // if c in -qc '-qc' is missing the argument, now opt is just 'c'
         opt = &opt[opt.len() - 1..];
-        streams.err.append(&wgettext_fmt!(
+        streams.err.appendln(&wgettext_fmt!(
             BUILTIN_ERR_MISSING,
             cmd,
             L!("-").to_owned() + opt
@@ -670,7 +692,7 @@ pub fn builtin_missing_argument(
     } else {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_MISSING, cmd, opt));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_MISSING, cmd, opt));
     }
     if print_hints {
         builtin_print_error_trailer(parser, streams.err, cmd);
@@ -687,7 +709,7 @@ pub fn builtin_unexpected_argument(
 ) {
     streams
         .err
-        .append(&wgettext_fmt!(BUILTIN_ERR_UNEXP_ARG, cmd, opt));
+        .appendln(&wgettext_fmt!(BUILTIN_ERR_UNEXP_ARG, cmd, opt));
     if print_hints {
         builtin_print_error_trailer(parser, streams.err, cmd);
     }
@@ -695,15 +717,14 @@ pub fn builtin_unexpected_argument(
 
 /// Print the backtrace and call for help that we use at the end of error messages.
 pub fn builtin_print_error_trailer(parser: &Parser, b: &mut OutputStream, cmd: &wstr) {
-    b.append_char('\n');
+    b.append('\n');
     let stacktrace = parser.current_line();
     // Don't print two empty lines if we don't have a stacktrace.
     if !stacktrace.is_empty() {
-        b.append(&stacktrace);
-        b.append_char('\n');
+        b.appendln(&stacktrace);
     }
-    b.append(&wgettext_fmt!(
-        "(Type 'help %s' for related documentation)\n",
+    b.appendln(&wgettext_fmt!(
+        "(Type 'help %s' for related documentation)",
         cmd
     ));
 }
@@ -715,9 +736,8 @@ pub fn builtin_wperror(program_name: &wstr, streams: &mut IoStreams) {
     streams.err.append(program_name);
     streams.err.append(L!(": "));
     if err.0 != 0 {
-        let werr = WString::from_str(&err.to_string());
-        streams.err.append(&werr);
-        streams.err.append_char('\n');
+        let werr = str2wcstring(err.to_string());
+        streams.err.appendln(&werr);
     }
 }
 
@@ -908,7 +928,7 @@ impl<'args, 'iter> Arguments<'args, 'iter> {
             return None;
         }
 
-        // assert!(num_bytes == self.buffer.len());
+        // assert_eq!(num_bytes, self.buffer.len());
         let (end, want_newline) = match (&self.split_behavior, buffer.last()) {
             // remove the newline — consumers do not expect it
             (Newline, Some(b'\n')) => (num_bytes - 1, true),
@@ -971,8 +991,8 @@ fn parsed_pid(
     match pid {
         Ok(pid @ 1..) => Ok(Pid::new(pid)),
         _ => {
-            streams.err.append(&wgettext_fmt!(
-                "%s: '%s' is not a valid process ID\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: '%s' is not a valid process ID",
                 cmd,
                 arg
             ));
@@ -1022,7 +1042,7 @@ pub fn builtin_break_continue(
     if argc != 1 {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_UNKNOWN, argv[0], argv[1]));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_UNKNOWN, argv[0], argv[1]));
         return Err(STATUS_INVALID_ARGS);
     }
 
@@ -1041,7 +1061,7 @@ pub fn builtin_break_continue(
     if !has_loop {
         streams
             .err
-            .append(&wgettext_fmt!("%s: Not inside of loop\n", argv[0]));
+            .appendln(&wgettext_fmt!("%s: Not inside of loop", argv[0]));
         return Err(STATUS_CMD_ERROR);
     }
 
@@ -1092,8 +1112,8 @@ impl ColorEnabled {
         arg: &wstr,
     ) -> Result<Self, ErrorCode> {
         Self::try_from(arg).map_err(|()| {
-            streams.err.append(&wgettext_fmt!(
-                "%s: Invalid value for '--color' option: '%s'. Expected 'always', 'never', or 'auto'\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "%s: Invalid value for '--color' option: '%s'. Expected 'always', 'never', or 'auto'",
                 cmd,
                 arg
             ));
