@@ -3,9 +3,10 @@ use std::collections::HashMap;
 use std::num::NonZeroUsize;
 
 use super::*;
+use crate::common::str2wcstring;
 use crate::env::{EnvVar, EnvVarFlags};
 use crate::flog::flog;
-use crate::parse_util::parse_util_unescape_wildcards;
+use crate::parse_util::unescape_wildcards;
 use crate::parser::ParserEnvSetMode;
 use crate::wildcard::{ANY_STRING, wildcard_match};
 
@@ -55,7 +56,7 @@ impl<'args> StringSubCommand<'args> for Match<'args> {
                         .and_then(|v| NonZeroUsize::new(v as usize))
                         .ok_or_else(|| {
                             StringError::InvalidArgs(wgettext_fmt!(
-                                "%s: Invalid max matches value '%s'\n",
+                                BUILTIN_ERR_INVALID_MAX_MATCHES,
                                 _n,
                                 arg
                             ))
@@ -94,7 +95,7 @@ impl<'args> StringSubCommand<'args> for Match<'args> {
         let cmd = args[0];
 
         if self.entire && self.index {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 cmd,
                 wgettext!("--entire and --index are mutually exclusive")
@@ -103,7 +104,7 @@ impl<'args> StringSubCommand<'args> for Match<'args> {
         }
 
         if self.invert_match && self.groups_only {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 cmd,
                 wgettext!("--invert and --groups-only are mutually exclusive")
@@ -112,7 +113,7 @@ impl<'args> StringSubCommand<'args> for Match<'args> {
         }
 
         if self.entire && self.groups_only {
-            streams.err.append(&wgettext_fmt!(
+            streams.err.appendln(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 cmd,
                 wgettext!("--entire and --groups-only are mutually exclusive")
@@ -130,7 +131,7 @@ impl<'args> StringSubCommand<'args> for Match<'args> {
 
         for InputValue { arg, .. } in arguments(args, optind, streams) {
             if let Err(e) = matcher.report_matches(arg.as_ref(), streams) {
-                flog!(error, "pcre2_match unexpected error:", e.error_message())
+                flog!(error, "pcre2_match unexpected error:", e.error_message());
             }
             let match_count = matcher.match_count();
             if self.quiet && match_count > 0
@@ -146,7 +147,7 @@ impl<'args> StringSubCommand<'args> for Match<'args> {
             ..
         }) = matcher
         {
-            for (name, vals) in first_match_captures.into_iter() {
+            for (name, vals) in first_match_captures {
                 parser.set_var(&WString::from(name), ParserEnvSetMode::default(), vals);
             }
         }
@@ -260,7 +261,7 @@ impl<'opts, 'args> RegexMatcher<'opts, 'args> {
                 Self::populate_captures_from_match(
                     &mut self.first_match_captures,
                     self.opts,
-                    actual,
+                    actual.as_ref(),
                 );
             }
         }
@@ -274,7 +275,7 @@ impl<'opts, 'args> RegexMatcher<'opts, 'args> {
                     Self::populate_captures_from_match(
                         &mut self.first_match_captures,
                         self.opts,
-                        &cg,
+                        cg.as_ref(),
                     );
                 }
             }
@@ -285,7 +286,7 @@ impl<'opts, 'args> RegexMatcher<'opts, 'args> {
     fn populate_captures_from_match<'a>(
         first_match_captures: &mut HashMap<String, Vec<WString>>,
         opts: &Match<'args>,
-        cg: &Option<Captures<'a>>,
+        cg: Option<&Captures<'a>>,
     ) {
         for (name, captures) in first_match_captures.iter_mut() {
             // If there are multiple named groups and --all was used, we need to ensure that
@@ -307,7 +308,7 @@ impl<'opts, 'args> RegexMatcher<'opts, 'args> {
         capture_group_names: &[Option<String>],
     ) -> Result<(), RegexError> {
         for name in capture_group_names.iter().filter_map(|n| n.as_ref()) {
-            let wname = WString::from_str(name);
+            let wname = str2wcstring(name);
             if EnvVar::flags_for(&wname).contains(EnvVarFlags::READ_ONLY) {
                 return Err(RegexError::InvalidCaptureGroupName(wname));
             }
@@ -365,7 +366,7 @@ impl<'opts, 'args> RegexMatcher<'opts, 'args> {
 
 impl<'opts, 'args> WildCardMatcher<'opts, 'args> {
     fn new(pattern: &'args wstr, opts: &'opts Match<'args>) -> Self {
-        let mut wcpattern = parse_util_unescape_wildcards(pattern);
+        let mut wcpattern = unescape_wildcards(pattern);
         if opts.ignore_case {
             wcpattern = wcpattern.to_lowercase();
         }
