@@ -1,15 +1,18 @@
-use std::cmp::Ordering;
+use std::{cmp::Ordering, sync::LazyLock};
 
 use libc::{RLIM_INFINITY, c_uint, rlim_t};
 use nix::errno::Errno;
 use nix::sys::resource::Resource as ResourceEnum;
-use once_cell::sync::Lazy;
 
 use crate::wutil::perror;
 use fish_fallback::{fish_wcswidth, wcscasecmp};
 
 use super::prelude::*;
 
+localizable_consts! {
+    BUILTIN_ULIMIT_UNLIMITED "unlimited"
+}
+
 pub mod limits {
     /// Constants that exist everywhere (except perhaps Cygwin).
     /// Note these are uints on Linux but ints everywhere else - we use -1 as a sentinel
@@ -21,7 +24,7 @@ pub mod limits {
         pub const DATA: libc::c_int = libc::RLIMIT_DATA as _;
         pub const FSIZE: libc::c_int = libc::RLIMIT_FSIZE as _;
         cfg_if!(
-            if #[cfg(cygwin)] {
+            if #[cfg(any(cygwin, target_os = "illumos"))] {
                 pub const MEMLOCK: libc::c_int = -1;
             } else {
                 pub const MEMLOCK: libc::c_int = libc::RLIMIT_MEMLOCK as _;
@@ -31,7 +34,7 @@ pub mod limits {
         pub const STACK: libc::c_int = libc::RLIMIT_STACK as _;
         pub const CPU: libc::c_int = libc::RLIMIT_CPU as _;
         cfg_if!(
-            if #[cfg(cygwin)] {
+            if #[cfg(any(cygwin, target_os = "illumos"))] {
                 pub const NPROC: libc::c_int = -1;
             } else {
                 pub const NPROC: libc::c_int = libc::RLIMIT_NPROC as _;
@@ -88,10 +91,9 @@ pub mod limits {
 
 fn convert_resource(resource: c_uint) -> ResourceEnum {
     let resource: i32 = resource.try_into().unwrap();
-    use std::mem::{size_of, transmute};
-    // Resource is #[repr(i32)] so this is ok
-    const _: () = assert!(size_of::<c_uint>() == size_of::<ResourceEnum>());
-    unsafe { transmute(resource) }
+
+    // SAFETY: Resource is #[repr(i32)] so this is sound
+    unsafe { std::mem::transmute(resource) }
 }
 
 /// Calls getrlimit.
@@ -108,16 +110,16 @@ fn setrlimit(resource: c_uint, rlim_cur: rlim_t, rlim_max: rlim_t) -> Result<(),
 /// Print the value of the specified resource limit.
 fn print(resource: c_uint, hard: bool, streams: &mut IoStreams) {
     let Some(l) = get(resource, hard) else {
-        streams.out.append(wgettext!("error\n"));
+        streams.out.appendln(wgettext!("error"));
         return;
     };
 
     if l == RLIM_INFINITY {
-        streams.out.append(wgettext!("unlimited\n"));
+        streams.out.appendln(wgettext!(BUILTIN_ULIMIT_UNLIMITED));
     } else {
         streams
             .out
-            .append(&wgettext_fmt!("%u\n", l / get_multiplier(resource)));
+            .appendln(&sprintf!("%u", l / get_multiplier(resource)));
     }
 }
 
@@ -150,12 +152,11 @@ fn print_all(hard: bool, streams: &mut IoStreams) {
         ));
 
         if l == RLIM_INFINITY {
-            streams.out.append(wgettext!("unlimited\n"));
+            streams.out.appendln(wgettext!(BUILTIN_ULIMIT_UNLIMITED));
         } else {
-            streams.out.append(&wgettext_fmt!(
-                "%u\n",
-                l / get_multiplier(resource.resource)
-            ));
+            streams
+                .out
+                .appendln(&sprintf!("%u", l / get_multiplier(resource.resource)));
         }
     }
 }
@@ -196,8 +197,8 @@ fn set_limit(
 
     if let Err(errno) = setrlimit(resource, rlim_cur, rlim_max) {
         if errno == Errno::EPERM {
-            streams.err.append(&wgettext_fmt!(
-                "ulimit: Permission denied when changing resource of type '%s'\n",
+            streams.err.appendln(&wgettext_fmt!(
+                "ulimit: Permission denied when changing resource of type '%s'",
                 get_desc(resource)
             ));
         } else {
@@ -333,8 +334,8 @@ pub fn ulimit(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
     }
 
     if opts.what == -1 {
-        streams.err.append(&wgettext_fmt!(
-            "%s: Resource limit not available on this operating system\n",
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: Resource limit not available on this operating system",
             cmd
         ));
 
@@ -352,7 +353,7 @@ pub fn ulimit(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
     } else if arg_count != 1 {
         streams
             .err
-            .append(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
+            .appendln(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
 
         builtin_print_error_trailer(parser, streams.err, cmd);
         return Err(STATUS_INVALID_ARGS);
@@ -365,10 +366,13 @@ pub fn ulimit(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         hard = true;
         soft = true;
     }
+    localizable_consts! {
+        BUILTIN_ULIMIT_INVALID "%s: Invalid limit '%s'"
+    }
 
     let new_limit: rlim_t = if w.wopt_index == argc {
-        streams.err.append(&wgettext_fmt!(
-            "%s: New limit cannot be an empty string\n",
+        streams.err.appendln(&wgettext_fmt!(
+            "%s: New limit cannot be an empty string",
             cmd
         ));
         builtin_print_error_trailer(parser, streams.err, cmd);
@@ -387,8 +391,8 @@ pub fn ulimit(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         }
     } else if let Ok(limit) = fish_wcstol(w.argv[w.wopt_index]) {
         let Some(x) = get_multiplier(what).checked_mul(limit as rlim_t) else {
-            streams.err.append(&wgettext_fmt!(
-                "%s: Invalid limit '%s'\n",
+            streams.err.appendln(&wgettext_fmt!(
+                BUILTIN_ULIMIT_INVALID,
                 cmd,
                 w.argv[w.wopt_index]
             ));
@@ -397,8 +401,8 @@ pub fn ulimit(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         };
         x
     } else {
-        streams.err.append(&wgettext_fmt!(
-            "%s: Invalid limit '%s'\n",
+        streams.err.appendln(&wgettext_fmt!(
+            BUILTIN_ULIMIT_INVALID,
             cmd,
             w.argv[w.wopt_index]
         ));
@@ -434,7 +438,7 @@ impl Resource {
 }
 
 /// Array of resource_t structs, describing all known resource types.
-static RESOURCE_ARR: Lazy<Box<[Resource]>> = Lazy::new(|| {
+static RESOURCE_ARR: LazyLock<Box<[Resource]>> = LazyLock::new(|| {
     let resources_info = [
         (
             limits::SBSIZE,
