@@ -2,7 +2,7 @@ use crate::common::{BUILD_DIR, get_program_name};
 use crate::{flog, flogf};
 use fish_build_helper::workspace_root;
 use std::ffi::OsStr;
-use std::os::unix::ffi::OsStrExt;
+use std::os::unix::ffi::OsStrExt as _;
 use std::path::{Path, PathBuf};
 use std::sync::OnceLock;
 
@@ -106,7 +106,7 @@ impl ConfigPaths {
         };
 
         let workspace_root = workspace_root();
-        // TODO(MSRV>=1.88): if-let-chain
+        // TODO(MSRV>=1.88): feature(let_chains)
         if cfg!(using_cmake) && exec_path_parent.ends_with("bin") && {
             let prefix = exec_path_parent.parent().unwrap();
             let data = prefix.join("share/fish");
@@ -135,18 +135,18 @@ impl ConfigPaths {
                     workspace_root.display()
                 ),
             );
-            let user_doc_join = |dir| {
+            let doc_join = |dir| {
                 cfg!(using_cmake)
                     .then_some(Path::new(BUILD_DIR))
-                    .map(|path| path.join("user_doc").join(dir))
+                    .map(|path| path.join("cargo").join("fish-docs").join(dir))
             };
             // If we're in Cargo's target directory or in CMake's build directory, use the source files.
             Self {
                 sysconf: workspace_root.join("etc"),
                 bin: Some(exec_path_parent.to_owned()),
                 data: Some(workspace_root.join("share")),
-                man: user_doc_join("man"),
-                doc: user_doc_join("html"),
+                man: doc_join("man"),
+                doc: doc_join("html"),
             }
         } else {
             flog!(
@@ -181,7 +181,6 @@ fn compute_fish_path() -> FishPath {
     // When /proc/self/exe points to a file that was deleted (or overwritten on update!)
     // then linux adds a " (deleted)" suffix.
     // If that's not a valid path, let's remove that awkward suffix.
-    // TODO(MSRV>=1.88) use if-let-chain
     if !path.exists() {
         if let (Some(filename), Some(parent)) = (path.file_name(), path.parent()) {
             if let Some(corrected_filename) = filename
