@@ -7,7 +7,7 @@
 use std::sync::atomic::{AtomicBool, AtomicU32, Ordering};
 use std::sync::{Arc, Mutex};
 
-use crate::common::{ScopeGuard, escape};
+use crate::common::{ScopeGuard, escape, str2wcstring};
 use crate::flog::flog;
 use crate::io::{IoChain, IoStreams};
 use crate::job_group::MaybeJobId;
@@ -380,7 +380,7 @@ pub fn get_desc(parser: &Parser, evt: &Event) -> WString {
             format!("signal handler for {} ({})", signal.name(), signal.desc(),)
         }
         EventDescription::Variable { name } => format!("handler for variable '{name}'"),
-        EventDescription::ProcessExit { pid: None } => "exit handler for any process".to_string(),
+        EventDescription::ProcessExit { pid: None } => "exit handler for any process".to_owned(),
         EventDescription::ProcessExit { pid: Some(pid) } => {
             format!("exit handler for process {pid}")
         }
@@ -392,17 +392,17 @@ pub fn get_desc(parser: &Parser, evt: &Event) -> WString {
                     format!("exit handler for job with pid {pid}")
                 }
             } else {
-                "exit handler for any job".to_string()
+                "exit handler for any job".to_owned()
             }
         }
         EventDescription::CallerExit { .. } => {
-            "exit handler for command substitution caller".to_string()
+            "exit handler for command substitution caller".to_owned()
         }
         EventDescription::Generic { param } => format!("handler for generic event '{param}'"),
         EventDescription::Any => unreachable!(),
     };
 
-    WString::from_str(&s)
+    str2wcstring(&s)
 }
 
 /// Add an event handler.
@@ -481,7 +481,7 @@ fn fire_internal(parser: &Parser, event: &Event) {
         // A previous handler may have erased this one.
         if handler.removed.load(Ordering::Relaxed) {
             continue;
-        };
+        }
 
         // Construct a buffer to evaluate, starting with the function name and then all the
         // arguments.
@@ -526,12 +526,12 @@ pub fn fire_delayed(parser: &Parser) {
     // Do not invoke new event handlers from within event handlers.
     if parser.scope().is_event {
         return;
-    };
+    }
 
     // Do not invoke new event handlers if we are unwinding (#6649).
     if signal_check_cancel() != 0 {
         return;
-    };
+    }
 
     // We unfortunately can't keep this locked until we're done with it because the SIGWINCH handler
     // code might call back into here and we would delay processing of the events, leading to a test
@@ -549,7 +549,7 @@ pub fn fire_delayed(parser: &Parser) {
         // HACK: The only variables we change in response to a *signal* are $COLUMNS and $LINES.
         // Do that now.
         if sig == libc::SIGWINCH {
-            reader_update_termsize(parser)
+            reader_update_termsize(parser);
         }
         let event = Event {
             desc: EventDescription::Signal { signal: sig },
@@ -664,5 +664,5 @@ pub fn fire_generic(parser: &Parser, name: WString, arguments: Vec<WString>) {
             desc: EventDescription::Generic { param: name },
             arguments,
         },
-    )
+    );
 }
