@@ -10,7 +10,7 @@ use crate::{common::exit_without_destructors, wutil::fstat};
 use libc::{O_RDONLY, pid_t};
 use std::ffi::CStr;
 use std::num::NonZeroU32;
-use std::os::unix::fs::MetadataExt;
+use std::os::unix::fs::MetadataExt as _;
 use std::time::Duration;
 
 /// The number of times to try to call fork() before giving up.
@@ -94,7 +94,7 @@ pub fn execute_setpgid(pid: libc::pid_t, pgroup: libc::pid_t, is_parent: bool) -
             return 0;
         }
         let err = errno::errno().0;
-        assert!(err != libc::EINTR);
+        assert_ne!(err, libc::EINTR);
         if err == libc::EACCES && is_parent {
             // We are the parent process and our child has called exec().
             // This is an unavoidable benign race.
@@ -503,7 +503,7 @@ fn get_interpreter<'a>(command: &CStr, buffer: &'a mut [u8]) -> Option<&'a CStr>
     if fd >= 0 {
         while idx + 1 < buffer.len() {
             let mut ch = b'\0';
-            let amt = unsafe { libc::read(fd, (&raw mut ch).cast(), std::mem::size_of_val(&ch)) };
+            let amt = unsafe { libc::read(fd, (&raw mut ch).cast(), size_of_val(&ch)) };
             if amt <= 0 || ch == b'\n' {
                 break;
             }
@@ -526,5 +526,23 @@ fn get_interpreter<'a>(command: &CStr, buffer: &'a mut [u8]) -> Option<&'a CStr>
     } else {
         return None;
     };
-    Some(CStr::from_bytes_with_nul(&buffer[offset..idx.max(offset)]).unwrap())
+    CStr::from_bytes_with_nul(&buffer[offset..idx.max(offset)]).ok()
+}
+
+#[cfg(test)]
+mod tests {
+    use super::get_interpreter;
+    use std::ffi::CString;
+    use std::os::unix::ffi::OsStrExt as _;
+
+    #[test]
+    fn test_get_interpreter_returns_none_on_embedded_nul() {
+        let script = fish_tempfile::new_file().unwrap();
+        std::fs::write(script.path(), b"#!/bin/\0sh\n").unwrap();
+
+        let command = CString::new(script.path().as_os_str().as_bytes()).unwrap();
+        let mut buffer = [0u8; 64];
+
+        assert!(get_interpreter(command.as_c_str(), &mut buffer).is_none());
+    }
 }
