@@ -1,5 +1,5 @@
 use crate::{
-    common::{bytes2wcstring, wcs2bytes, wcs2osstring},
+    common::{osstr2wcstring, wcs2bytes, wcs2osstring},
     fds::wopen_cloexec,
     flog, flogf,
     path::{DirRemoteness, path_remoteness},
@@ -15,8 +15,8 @@ use std::{
     ffi::OsString,
     fs::{File, OpenOptions},
     os::{
-        fd::AsRawFd,
-        unix::{ffi::OsStringExt, fs::MetadataExt},
+        fd::AsRawFd as _,
+        unix::{ffi::OsStringExt as _, fs::MetadataExt as _},
     },
     path::PathBuf,
 };
@@ -32,7 +32,7 @@ fn create_temporary_file(original_path: &wstr) -> std::io::Result<(File, WString
     let (path, result) =
         fish_tempfile::create_file_with_retry(|| dir.join(random_filename(prefix.clone())));
     match result {
-        Ok(file) => Ok((file, bytes2wcstring(path.as_os_str().as_encoded_bytes()))),
+        Ok(file) => Ok((file, osstr2wcstring(path))),
         Err(e) => {
             flog!(
                 error,
@@ -340,13 +340,12 @@ where
 
     /// Renames a file from `old_name` to `new_name`.
     fn rename(old_name: &wstr, new_name: &wstr) -> std::io::Result<()> {
-        if wrename(old_name, new_name) == -1 {
-            let error_number = errno::errno();
+        if let Err(e) = wrename(old_name, new_name) {
             flog!(
                 error,
-                wgettext_fmt!("Error when renaming file: %s", error_number.to_string())
+                wgettext_fmt!("Error when renaming file: %s", e.to_string())
             );
-            return Err(std::io::Error::from(error_number));
+            return Err(e);
         }
         Ok(())
     }
@@ -478,7 +477,7 @@ where
             .as_ref()
             .is_ok_and(|(_file_id, potential_update)| !potential_update.do_save)
     {
-        wunlink(&tmp_name);
+        let _ = wunlink(&tmp_name);
     }
     result
 }
