@@ -13,7 +13,7 @@ use crate::threads::assert_is_main_thread;
 use fish_common::assert_sorted_by_name;
 use std::mem;
 use std::sync::{
-    LazyLock, Mutex, MutexGuard,
+    Mutex, MutexGuard,
     atomic::{AtomicU32, Ordering},
 };
 
@@ -162,6 +162,7 @@ const INPUT_FUNCTION_METADATA: &[InputFunctionMetadata] = &[
     make_md(L!("forward-word"), ReadlineCmd::ForwardWordEmacs),
     make_md(L!("forward-word-end"), ReadlineCmd::ForwardWordEnd),
     make_md(L!("forward-word-vi"), ReadlineCmd::ForwardWordVi),
+    make_md(L!("get-key"), ReadlineCmd::GetKey),
     make_md(L!("history-delete"), ReadlineCmd::HistoryDelete),
     make_md(L!("history-last-token-search-backward"), ReadlineCmd::HistoryLastTokenSearchBackward),
     make_md(L!("history-last-token-search-forward"), ReadlineCmd::HistoryLastTokenSearchForward),
@@ -238,10 +239,18 @@ pub struct InputMappingSet {
     preset_mapping_list: Vec<InputMapping>,
 }
 
+impl InputMappingSet {
+    const fn new() -> Self {
+        Self {
+            mapping_list: Vec::new(),
+            preset_mapping_list: Vec::new(),
+        }
+    }
+}
+
 /// Access the singleton input mapping set.
 pub fn input_mappings() -> MutexGuard<'static, InputMappingSet> {
-    static INPUT_MAPPINGS: LazyLock<Mutex<InputMappingSet>> =
-        LazyLock::new(|| Mutex::new(InputMappingSet::default()));
+    static INPUT_MAPPINGS: Mutex<InputMappingSet> = Mutex::new(InputMappingSet::new());
     INPUT_MAPPINGS.lock().unwrap()
 }
 
@@ -698,7 +707,9 @@ impl<'a> Reader<'a> {
             let evt = self.readch();
             match evt {
                 CharEvent::Readline(ref readline_event) => match readline_event.cmd {
-                    ReadlineCmd::SelfInsert | ReadlineCmd::SelfInsertNotFirst => {
+                    ReadlineCmd::SelfInsert
+                    | ReadlineCmd::SelfInsertNotFirst
+                    | ReadlineCmd::GetKey => {
                         // Typically self-insert is generated by the generic (empty) binding.
                         // However if it is generated by a real sequence, then insert that sequence.
                         let seq = readline_event.seq.chars().map(CharEvent::from_char);
@@ -721,6 +732,20 @@ impl<'a> Reader<'a> {
                                 kevt.input_style = CharInputStyle::NotFirst;
                             }
                         }
+                        if readline_event.cmd == ReadlineCmd::GetKey {
+                            if let CharEvent::Key(kevt) = res {
+                                return CharEvent::Command(sprintf!(
+                                    "set -g fish_key %s",
+                                    escape(
+                                        &kevt
+                                            .key
+                                            .codepoint_text()
+                                            .map(|c| WString::from_chars(vec![c]))
+                                            .unwrap_or_default()
+                                    )
+                                ));
+                            }
+                        }
                         return res;
                     }
                     ReadlineCmd::FuncAnd | ReadlineCmd::FuncOr => {
