@@ -11,7 +11,6 @@ use crate::prelude::*;
 use crate::proc::JobGroupRef;
 use crate::redirection::{RedirectionMode, RedirectionSpecList};
 use crate::signal::SigChecker;
-use crate::terminal::Output;
 use crate::topic_monitor::Topic;
 use crate::wutil::{perror, perror_io, unescape_bytes_and_write_to_fd, wdirname, wstat};
 use errno::Errno;
@@ -20,7 +19,7 @@ use nix::fcntl::OFlag;
 use nix::sys::stat::Mode;
 use std::fs::File;
 use std::io;
-use std::os::fd::{AsFd, AsRawFd, BorrowedFd, OwnedFd, RawFd};
+use std::os::fd::{AsFd as _, AsRawFd as _, BorrowedFd, OwnedFd, RawFd};
 use std::sync::{Arc, LazyLock, Mutex, MutexGuard};
 
 /// separated_buffer_t represents a buffer of output from commands, prepared to be turned into a
@@ -732,13 +731,6 @@ impl OutputStream {
     }
 }
 
-impl Output for OutputStream {
-    fn write_bytes(&mut self, command_part: &[u8]) {
-        // TODO Retry on interrupt.
-        self.append(&bytes2wcstring(command_part));
-    }
-}
-
 /// An output stream for builtins which outputs to an fd.
 /// Note the fd may be something like stdout; there is no ownership implied here.
 pub struct FdOutputStream {
