@@ -1,7 +1,8 @@
 //! Provides the "linkage" between an ast and actual execution structures (job_t, etc.).
 
 use crate::ast::{
-    self, BlockStatementHeader, Keyword, Leaf, Node, Statement, Token, unescape_keyword,
+    self, BlockStatementHeader, Keyword as _, Leaf as _, Node, Statement, Token as _,
+    unescape_keyword,
 };
 use crate::builtins;
 use crate::builtins::shared::{
@@ -13,7 +14,7 @@ use crate::common::{
     ScopeGuard, ScopeGuarding, ScopedRefCell, escape, truncate_at_nul, valid_var_name,
 };
 use crate::complete::CompletionList;
-use crate::env::{EnvMode, EnvStackSetResult, EnvVar, EnvVarFlags, Environment, Statuses};
+use crate::env::{EnvMode, EnvStackSetResult, EnvVar, EnvVarFlags, Environment as _, Statuses};
 use crate::event::{self, Event};
 use crate::exec::exec_job;
 use crate::expand::{
@@ -30,10 +31,9 @@ use crate::parse_constants::{
     ParseError, ParseErrorCode, ParseErrorList, ParseKeyword, ParseTokenType, StatementDecoration,
     parse_error_offset_source_start,
 };
-use crate::parse_tree::{LineCounter, NodeRef, ParsedSourceRef};
+use crate::parse_tree::{NodeRef, ParsedSourceRef};
 use crate::parse_util::{
-    MaybeParentheses::CommandSubstitution, parse_util_locate_cmdsubst_range,
-    parse_util_unescape_wildcards,
+    MaybeParentheses::CommandSubstitution, locate_cmdsubst_range, unescape_wildcards,
 };
 use crate::parser::{
     Block, BlockData, BlockId, BlockType, LoopStatus, Parser, ParserEnvSetMode, ProfileItem,
@@ -52,7 +52,8 @@ use crate::timer::push_timer;
 use crate::tokenizer::{PipeOrRedir, TokenType, variable_assignment_equals_pos};
 use crate::trace::{trace_if_enabled, trace_if_enabled_with_args};
 use crate::wildcard::wildcard_match;
-use fish_wchar::WExt;
+use fish_common::help_section;
+use fish_widestring::WExt as _;
 use libc::{ENOTDIR, EXIT_SUCCESS, STDERR_FILENO, STDOUT_FILENO, c_int};
 use std::io::ErrorKind;
 use std::rc::Rc;
@@ -61,7 +62,7 @@ use std::sync::Arc;
 /// An eval_result represents evaluation errors including wildcards which failed to match, syntax
 /// errors, or other expansion errors. It also tracks when evaluation was skipped due to signal
 /// cancellation. Note it does not track the exit status of commands.
-#[derive(Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq)]
 pub enum EndExecutionReason {
     /// Evaluation was successful.
     Ok,
@@ -84,9 +85,9 @@ pub struct ExecutionContext<'a> {
     // unwinding.
     cancel_signal: Option<Signal>,
 
-    // Helper to count lines.
+    // The currently executing pipeline node.
     // This is shared with the Parser so that the Parser can access the current line.
-    line_counter: &'a ScopedRefCell<LineCounter<ast::JobPipeline>>,
+    pipeline_node: &'a ScopedRefCell<Option<NodeRef<ast::JobPipeline>>>,
 
     /// The block IO chain.
     /// For example, in `begin; foo ; end < file.txt` this would have the 'file.txt' IO.
@@ -117,12 +118,14 @@ macro_rules! report_error_formatted {
 }
 
 pub fn varname_error(command: &wstr, bad_name: &wstr) -> WString {
-    wgettext_fmt!(
+    let mut e = wgettext_fmt!(
         BUILTIN_ERR_VARNAME,
         command,
         bad_name,
         help_section!("language#shell-variable-and-function-names")
-    )
+    );
+    e.push('\n');
+    e
 }
 
 impl<'a> ExecutionContext<'a> {
@@ -131,13 +134,13 @@ impl<'a> ExecutionContext<'a> {
     pub fn new(
         pstree: ParsedSourceRef,
         block_io: IoChain,
-        line_counter: &'a ScopedRefCell<LineCounter<ast::JobPipeline>>,
+        pipeline_node: &'a ScopedRefCell<Option<NodeRef<ast::JobPipeline>>>,
         test_only_suppress_stderr: bool,
     ) -> Self {
         Self {
             pstree,
             cancel_signal: None,
-            line_counter,
+            pipeline_node,
             block_io,
             test_only_suppress_stderr,
         }
@@ -779,7 +782,7 @@ impl<'a> ExecutionContext<'a> {
                         std::io::Error::from_raw_os_error(external_cmd.err.unwrap().into()),
                     );
                 }
-            };
+            }
             path
         } else {
             WString::new()
@@ -936,7 +939,7 @@ impl<'a> ExecutionContext<'a> {
             ParserEnvSetMode::user(EnvMode::LOCAL),
             var.map_or(vec![], |var| var.as_list().to_owned()),
         );
-        assert!(retval == EnvStackSetResult::Ok);
+        assert_eq!(retval, EnvStackSetResult::Ok);
 
         trace_if_enabled_with_args(ctx.parser(), L!("for"), &arguments);
 
@@ -956,8 +959,9 @@ impl<'a> ExecutionContext<'a> {
                 ParserEnvSetMode::user(EnvMode::empty()),
                 vec![val],
             );
-            assert!(
-                retval == EnvStackSetResult::Ok,
+            assert_eq!(
+                retval,
+                EnvStackSetResult::Ok,
                 "for loop variable should have been successfully set"
             );
             event::fire(ctx.parser(), evt.clone());
@@ -1105,13 +1109,17 @@ impl<'a> ExecutionContext<'a> {
             }
             ExpandResultCode::ok => {
                 if switch_values_expanded.len() > 1 {
-                    return report_error!(
+                    let mut error = wgettext_fmt!(
+                        "switch: Expected at most one argument, got %u",
+                        switch_values_expanded.len()
+                    );
+                    error.push('\n');
+                    return report_error_formatted!(
                         self,
                         ctx,
                         STATUS_INVALID_ARGS,
                         &statement.argument,
-                        "switch: Expected at most one argument, got %u\n",
-                        switch_values_expanded.len()
+                        error
                     );
                 }
             }
@@ -1155,7 +1163,7 @@ impl<'a> ExecutionContext<'a> {
             if case_result == EndExecutionReason::Ok {
                 for arg in case_args {
                     // Unescape wildcards so they can be expanded again.
-                    let unescaped_arg = parse_util_unescape_wildcards(&arg);
+                    let unescaped_arg = unescape_wildcards(&arg);
                     if wildcard_match(&switch_value_expanded, &unescaped_arg, false) {
                         // If this matched, we're done.
                         matching_case_item = Some(case_item);
@@ -1170,7 +1178,7 @@ impl<'a> ExecutionContext<'a> {
 
         if let Some(case_item) = matching_case_item {
             // Success, evaluate the job list.
-            assert!(result == EndExecutionReason::Ok, "Expected success");
+            assert_eq!(result, EndExecutionReason::Ok, "Expected success");
             result = self.run_job_list(ctx, &case_item.body, Some(sb));
         }
 
@@ -1462,7 +1470,7 @@ impl<'a> ExecutionContext<'a> {
                 if oper.mode == RedirectionMode::Input && {
                     let redir_unexpanded = self.node_source(redir_node);
                     redir_unexpanded.starts_with(L!("<("))
-                        && match parse_util_locate_cmdsubst_range(
+                        && match locate_cmdsubst_range(
                             &redir_unexpanded[1..],
                             &mut 0,
                             false,
@@ -1529,9 +1537,8 @@ impl<'a> ExecutionContext<'a> {
         let _saved_eval_level = ctx.parser().push_scope(|s| s.eval_level += 1);
 
         // Save the executing node.
-        let _saved_node = self
-            .line_counter
-            .scoped_set(std::ptr::from_ref(job_node), |s| &mut s.node);
+        let executing_node = NodeRef::new(Arc::clone(self.pstree()), job_node);
+        let _saved_node = self.pipeline_node.scoped_replace(Some(executing_node));
 
         // Profiling support.
         let profile_item_id = ctx.parser().create_profile_item();
