@@ -36,12 +36,12 @@ use crate::wutil::perror;
 use crate::{flog, flogf, function};
 use assert_matches::assert_matches;
 use fish_util::get_time;
-use fish_widestring::WExt;
+use fish_widestring::WExt as _;
 use libc::c_int;
 use std::cell::{Ref, RefCell, RefMut};
 use std::ffi::OsStr;
 use std::fs::File;
-use std::io::Write;
+use std::io::Write as _;
 use std::num::NonZeroU32;
 use std::os::fd::OwnedFd;
 use std::rc::Rc;
@@ -791,16 +791,15 @@ impl Parser {
         if !self.is_interactive() || self.is_function() {
             if let Some(file) = file {
                 prefix.push_utfstr(&wgettext_fmt!(
-                    "%s (line %d):",
+                    "%s (line %d)",
                     &user_presentable_path(&file, self.vars()),
                     lineno
                 ));
             } else if self.libdata().within_fish_init {
-                prefix.push_utfstr(&wgettext_fmt!("Startup (line %d):", lineno));
+                prefix.push_utfstr(&wgettext_fmt!("Startup (line %d)", lineno));
             } else {
-                prefix.push_utfstr(&wgettext_fmt!("Standard input (line %d):", lineno));
+                prefix.push_utfstr(&wgettext_fmt!("Standard input (line %d)", lineno));
             }
-            prefix.push(' ');
         }
 
         let skip_caret = self.is_interactive() && !self.is_function();
@@ -1191,18 +1190,16 @@ impl Parser {
 
         let prefix = if let Some(filename) = self.current_filename() {
             if which_line > 0 {
-                let mut prefix = wgettext_fmt!(
-                    "%s (line %u):",
+                wgettext_fmt!(
+                    "%s (line %u)",
                     user_presentable_path(&filename, self.vars()),
                     which_line
-                );
-                prefix.push(' ');
-                prefix
+                )
             } else {
-                sprintf!("%s: ", user_presentable_path(&filename, self.vars()))
+                user_presentable_path(&filename, self.vars())
             }
         } else {
-            L!("fish: ").to_owned()
+            L!("fish").to_owned()
         };
 
         let mut output = err.describe_with_prefix(src, &prefix, self.is_interactive(), skip_caret);
