@@ -36,7 +36,7 @@ use nix::{
 };
 use std::cell::{Cell, Ref, RefCell, RefMut};
 use std::fs;
-use std::io::{Read, Write};
+use std::io::{Read as _, Write as _};
 use std::num::NonZeroU32;
 use std::os::fd::RawFd;
 use std::rc::Rc;
@@ -1465,7 +1465,7 @@ fn summary_command(j: &Job, p: Option<&Process>) -> WString {
             if j.external_procs().count() > 1 {
                 // I don't think it's safe to blindly unwrap here because even though we exited with
                 // a signal, the job could have contained a fish function?
-                let pid = p.pid().map_or("-".to_string(), |p| p.to_string());
+                let pid = p.pid().map_or("-".to_owned(), |p| p.to_string());
                 buffer += &sprintf!(" %s", pid)[..];
 
                 buffer.push(' ');
