@@ -12,14 +12,15 @@ use crate::key::ViewportPosition;
 use crate::pager::{PAGER_MIN_HEIGHT, PageRendering, Pager};
 use std::cell::RefCell;
 use std::collections::LinkedList;
-use std::io::Write;
+use std::io::Write as _;
 use std::num::NonZeroU16;
 use std::ops::Range;
 use std::sync::Mutex;
 use std::sync::atomic::AtomicU32;
 use std::time::SystemTime;
 
-use libc::{ONLCR, STDERR_FILENO, STDOUT_FILENO};
+use libc::{STDERR_FILENO, STDOUT_FILENO};
+use nix::sys::termios;
 
 use crate::common::{
     get_ellipsis_char, get_omitted_newline_str, has_working_tty_timestamps, shell_modes, wcs2bytes,
@@ -30,15 +31,16 @@ use crate::flog::{flog, flogf};
 use crate::global_safety::RelaxedAtomicBool;
 use crate::highlight::{HighlightColorResolver, HighlightRole, HighlightSpec};
 use crate::prelude::*;
+use crate::terminal::SgrTerminalCommand::EnterDimMode;
 use crate::terminal::TerminalCommand::{
     self, ClearToEndOfLine, ClearToEndOfScreen, CursorDown, CursorLeft, CursorMove, CursorRight,
-    CursorUp, EnterDimMode, ExitAttributeMode, Osc133PromptEnd, Osc133PromptStart, ScrollContentUp,
+    CursorUp, Osc133PromptEnd, Osc133PromptStart, ScrollContentUp,
 };
-use crate::terminal::{BufferedOutputter, CardinalDirection, Output, Outputter, use_terminfo};
+use crate::terminal::{BufferedOutputter, CardinalDirection, Outputter};
 use crate::termsize::Termsize;
-use crate::wcstringutil::{fish_wcwidth_visible, string_prefixes_string};
 use crate::wutil::fstat;
 use fish_fallback::fish_wcwidth;
+use fish_wcstringutil::{fish_wcwidth_visible, string_prefixes_string};
 
 #[derive(Copy, Clone, Default)]
 pub enum CharOffset {
@@ -87,7 +89,7 @@ impl Line {
             highlight,
             character: rendered_character(character),
             offset_in_cmdline,
-        })
+        });
     }
 
     /// Append a nul-terminated string `txt` to the line, giving each character `color`.
@@ -169,16 +171,16 @@ impl ScreenData {
     }
 
     pub fn clear_lines(&mut self) {
-        self.line_datas.clear()
+        self.line_datas.clear();
     }
 
     pub fn resize(&mut self, size: usize) {
-        self.line_datas.resize(size, Default::default())
+        self.line_datas.resize(size, Default::default());
     }
 
     pub fn create_line(&mut self, idx: usize) -> &mut Line {
         if idx >= self.line_datas.len() {
-            self.line_datas.resize(idx + 1, Default::default())
+            self.line_datas.resize(idx + 1, Default::default());
         }
         self.line_mut(idx)
     }
@@ -530,7 +532,7 @@ impl Screen {
                 });
 
         self.with_buffered_output(|zelf| {
-            zelf.update(vars, &layout.left_prompt, &layout.right_prompt)
+            zelf.update(vars, &layout.left_prompt, &layout.right_prompt);
         });
         self.save_status();
     }
@@ -569,7 +571,9 @@ impl Screen {
         }
         let mut out = BufferedOutputter::new(self.outp);
         // Move everything to scrollback.
-        out.write_command(ScrollContentUp(lines_to_scroll));
+        out.write_command(ScrollContentUp {
+            lines: lines_to_scroll,
+        });
         // Reposition cursor.
         out.write_command(CursorMove(CardinalDirection::Up, lines_to_scroll));
         self.set_position_in_viewport("scrollback-push", Some(0));
@@ -579,7 +583,7 @@ impl Screen {
         flogf!(
             reader,
             "Setting screen y to %s due to %s",
-            viewport_y.map_or("<none>".to_string(), |y| format!("{y}")),
+            viewport_y.map_or("<none>".to_owned(), |y| format!("{y}")),
             whence,
         );
         self.viewport_y = viewport_y;
@@ -685,7 +689,7 @@ fn abandon_line_string(screen_width: Option<usize>) -> Vec<u8> {
         return vec![b'\r'];
     };
 
-    let mut abandon_line_string = Vec::with_capacity(screen_width + 32);
+    let mut abandon_line_string = Outputter::new_buffering_no_assume_normal();
 
     let omitted_newline_str = get_omitted_newline_str();
 
@@ -695,71 +699,32 @@ fn abandon_line_string(screen_width: Option<usize>) -> Vec<u8> {
     let non_space_width = omitted_newline_str.chars().count();
     // We do `>` rather than `>=` because the code below might require one extra space.
     if screen_width > non_space_width {
-        if use_terminfo() {
-            use crate::terminal::tparm1;
-            use std::ffi::CString;
-            let term = crate::terminal::term();
-            let mut justgrey = true;
-            let add = |abandon_line_string: &mut Vec<u8>, s: Option<CString>| {
-                let Some(s) = s else {
-                    return false;
-                };
-                abandon_line_string.extend(s.as_bytes());
-                true
-            };
-            if let Some(enter_dim_mode) = term.enter_dim_mode.as_ref() {
-                if add(&mut abandon_line_string, Some(enter_dim_mode.clone())) {
-                    // Use dim if they have it, so the color will be based on their actual normal
-                    // color and the background of the terminal.
-                    justgrey = false;
-                }
-            }
-            if let (true, Some(set_a_foreground)) = (justgrey, term.set_a_foreground.as_ref()) {
-                let max_colors = term.max_colors.unwrap_or_default();
-                if max_colors >= 238 {
-                    // draw the string in a particular grey
-                    add(&mut abandon_line_string, tparm1(set_a_foreground, 237));
-                } else if max_colors >= 9 {
-                    // bright black (the ninth color, looks grey)
-                    add(&mut abandon_line_string, tparm1(set_a_foreground, 8));
-                } else if max_colors >= 2 {
-                    if let Some(enter_bold_mode) = term.enter_bold_mode.as_ref() {
-                        // we might still get that color by setting black and going bold for bright
-                        add(&mut abandon_line_string, Some(enter_bold_mode.clone()));
-                        add(&mut abandon_line_string, tparm1(set_a_foreground, 0));
-                    }
-                }
-            }
-        } else {
-            abandon_line_string.write_command(EnterDimMode);
-        }
+        abandon_line_string
+            .style_writer()
+            .write_command(EnterDimMode);
 
-        abandon_line_string.extend_from_slice(omitted_newline_str.as_bytes());
-        abandon_line_string.write_command(ExitAttributeMode);
-        abandon_line_string.extend(repeat_n(b' ', screen_width - non_space_width));
+        abandon_line_string.write_bytes(omitted_newline_str.as_bytes());
+        abandon_line_string.reset_text_face();
+        abandon_line_string
+            .contents_mut()
+            .extend(repeat_n(b' ', screen_width - non_space_width));
     }
 
     abandon_line_string.push(b'\r');
-    abandon_line_string.extend_from_slice(omitted_newline_str.as_bytes());
+    abandon_line_string.write_bytes(omitted_newline_str.as_bytes());
     // Now we are certainly on a new line. But we may have dropped the omitted newline char on
     // it. So append enough spaces to overwrite the omitted newline char, and then clear all the
     // spaces from the new line.
-    abandon_line_string.extend(repeat_n(b' ', non_space_width));
-    abandon_line_string.push(b'\r');
-    // Clear entire line. Zsh doesn't do this. Fish added this with commit 4417a6ee: If you have
-    // a prompt preceded by a new line, you'll get a line full of spaces instead of an empty
-    // line above your prompt. This doesn't make a difference in normal usage, but copying and
-    // pasting your terminal log becomes a pain. This commit clears that line, making it an
-    // actual empty line.
-    abandon_line_string.write_command(ClearToEndOfLine);
-    abandon_line_string.push(b'\r');
-    // Clear entire line. Zsh doesn't do this. Fish added this with commit 4417a6ee: If you have
-    // a prompt preceded by a new line, you'll get a line full of spaces instead of an empty
-    // line above your prompt. This doesn't make a difference in normal usage, but copying and
-    // pasting your terminal log becomes a pain. This commit clears that line, making it an
-    // actual empty line.
-    abandon_line_string.write_command(ClearToEndOfLine);
     abandon_line_string
+        .contents_mut()
+        .extend(repeat_n(b' ', non_space_width));
+    abandon_line_string.push(b'\r');
+    // Clear entire line. Zsh doesn't do this. Fish added this with commit 4417a6ee: If you have
+    // a prompt preceded by a new line, you'll get a line full of spaces instead of an empty
+    // line above your prompt. This doesn't make a difference in normal usage, but copying and
+    // pasting your terminal log becomes a pain. This commit clears that line, making it an
+    // actual empty line.
+    abandon_line_string.take_contents()
 }
 
 impl Screen {
@@ -923,16 +888,13 @@ impl Screen {
         let y_steps =
             isize::try_from(new_y).unwrap() - isize::try_from(self.actual.cursor.y).unwrap();
 
-        #[allow(clippy::comparison_chain)] // TODO(MSRV>=1.90) for old clippy
+        #[allow(clippy::comparison_chain)] // TODO(MSRV>=1.90) old clippy
         let s = if y_steps < 0 {
             Some(CursorUp)
         } else if y_steps > 0 {
-            if (shell_modes().c_oflag & ONLCR) != 0
-                && (!use_terminfo()
-                    || crate::terminal::term()
-                        .cursor_down
-                        .as_ref()
-                        .is_some_and(|cud| cud.as_bytes() == b"\n"))
+            if shell_modes()
+                .output_flags
+                .contains(termios::OutputFlags::ONLCR)
             {
                 // See GitHub issue #4505.
                 // Most consoles use a simple newline as the cursor down escape.
@@ -978,7 +940,7 @@ impl Screen {
                     x_steps.abs_diff(0),
                 ));
             }
-        };
+        }
 
         self.actual.cursor.x = new_x;
         self.actual.cursor.y = new_y;
@@ -1033,8 +995,7 @@ impl Screen {
     }
 
     fn should_wrap(&self, i: usize) -> bool {
-        allow_soft_wrap()
-            && self.desired.line(i).is_soft_wrapped
+        self.desired.line(i).is_soft_wrapped
             && i + 1 < self.desired.line_count()
             && (i + 1 >= self.actual.line_count()
                 || line_shared_prefix(self.actual.line(i + 1), self.desired.line(i + 1)) == 0)
@@ -1421,9 +1382,7 @@ pub struct LayoutCache {
 }
 
 // Singleton of the cached escape sequences seen in prompts and similar strings.
-// Note this is deliberately exported so that init_terminal can clear it.
-// TODO un-export this once we remove the terminfo option.
-pub static LAYOUT_CACHE_SHARED: Mutex<LayoutCache> = Mutex::new(LayoutCache::new());
+static LAYOUT_CACHE_SHARED: Mutex<LayoutCache> = Mutex::new(LayoutCache::new());
 
 impl LayoutCache {
     pub const fn new() -> Self {
@@ -1592,8 +1551,7 @@ pub fn escape_code_length(code: &wstr) -> Option<usize> {
         return None;
     }
 
-    is_terminfo_escape_seq(code)
-        .or_else(|| is_screen_name_escape_seq(code))
+    is_screen_name_escape_seq(code)
         .or_else(|| is_osc_escape_seq(code))
         .or_else(|| is_three_byte_escape_seq(code))
         .or_else(|| is_csi_style_escape_seq(code))
@@ -1606,42 +1564,17 @@ static MIDNIGHT_COMMANDER_HACK: RelaxedAtomicBool = RelaxedAtomicBool::new(false
 /// broken if you do '\r' after it like we normally do.
 /// See <https://midnight-commander.org/ticket/4258>.
 pub fn screen_set_midnight_commander_hack() {
-    MIDNIGHT_COMMANDER_HACK.store(true)
+    MIDNIGHT_COMMANDER_HACK.store(true);
 }
 
 /// The number of characters to indent new blocks.
 const INDENT_STEP: usize = 4;
 
-/// Tests if the specified narrow character sequence is present at the specified position of the
-/// specified wide character string. All of \c seq must match, but str may be longer than seq.
-fn try_sequence(seq: &[u8], s: &wstr) -> usize {
-    let mut i = 0;
-    loop {
-        if i == seq.len() {
-            return i;
-        }
-        if char::from(seq[i]) != s.char_at(i) {
-            return 0;
-        }
-        i += 1;
-    }
-}
-
 /// Returns the number of columns left until the next tab stop, given the current cursor position.
 fn next_tab_stop(current_line_width: usize) -> usize {
     // Assume tab stops every 8 characters.
-    let tab_width = if use_terminfo() {
-        crate::terminal::term().init_tabs.unwrap_or(8)
-    } else {
-        8
-    };
-    ((current_line_width / tab_width) + 1) * tab_width
-}
-
-/// Whether we permit soft wrapping. If so, in some cases we don't explicitly move to the second
-/// physical line on a wrapped logical line; instead we just output it.
-fn allow_soft_wrap() -> bool {
-    !use_terminfo() || crate::terminal::term().auto_right_margin
+    const TAB_WIDTH: usize = 8;
+    ((current_line_width / TAB_WIDTH) + 1) * TAB_WIDTH
 }
 
 /// Does this look like the escape sequence for setting a screen name?
@@ -1756,38 +1689,6 @@ fn is_csi_style_escape_seq(code: &wstr) -> Option<usize> {
     Some(cursor)
 }
 
-/// Detect whether the escape sequence sets one of the terminal attributes that affects how text is
-/// displayed other than the color.
-fn is_terminfo_escape_seq(code: &wstr) -> Option<usize> {
-    if !use_terminfo() {
-        return None;
-    }
-    let term = crate::terminal::term();
-    let esc2 = [
-        &term.enter_bold_mode,
-        &term.exit_attribute_mode,
-        &term.enter_underline_mode,
-        &term.exit_underline_mode,
-        &term.enter_standout_mode,
-        &term.enter_italics_mode,
-        &term.exit_italics_mode,
-        &term.enter_reverse_mode,
-        &term.enter_dim_mode,
-    ];
-
-    for p in &esc2 {
-        let Some(p) = p else { continue };
-        // Test both padded and unpadded version, just to be safe. Most versions of fish_tparm don't
-        // actually seem to do anything these days.
-        let esc_seq_len = try_sequence(p.as_bytes(), code);
-        if esc_seq_len != 0 {
-            return Some(esc_seq_len);
-        }
-    }
-
-    None
-}
-
 /// Return whether `c` ends a measuring run.
 fn is_run_terminator(c: char) -> bool {
     matches!(c, '\0' | '\n' | '\r' | '\x0C')
@@ -1923,13 +1824,6 @@ pub(crate) static ONLY_GRAYSCALE: RelaxedAtomicBool = RelaxedAtomicBool::new(fal
 
 /// Returns true if we are using a dumb terminal.
 pub(crate) fn is_dumb() -> bool {
-    if use_terminfo() {
-        let term = crate::terminal::term();
-        return term.cursor_up.is_none()
-            || term.cursor_down.is_none()
-            || term.cursor_left.is_none()
-            || term.cursor_right.is_none();
-    }
     IS_DUMB.load()
 }
 
@@ -2185,17 +2079,17 @@ pub fn wcswidth_rendered(s: &wstr) -> isize {
 mod tests {
     use crate::common::get_ellipsis_char;
     use crate::highlight::HighlightSpec;
-    use crate::parse_util::parse_util_compute_indents;
+    use crate::parse_util::compute_indents;
     use crate::prelude::*;
     use crate::screen::{
         LayoutCache, PromptCacheEntry, PromptLayout, ScreenLayout, compute_layout,
     };
     use crate::tests::prelude::*;
-    use crate::wcstringutil::join_strings;
+    use fish_wcstringutil::join_strings;
 
     #[test]
     #[serial]
-    fn test_complete() {
+    fn test_escape_code_length() {
         let _cleanup = test_init();
         let mut lc = LayoutCache::new();
         assert_eq!(lc.escape_code_length(L!("")), 0);
@@ -2448,7 +2342,7 @@ mod tests {
                     + L!($autosuggestion_str)
                     + L!($commandline_after_suggestion);
                 let mut colors = vec![HighlightSpec::default(); full_commandline.len()];
-                let mut indent = parse_util_compute_indents(&full_commandline);
+                let mut indent = compute_indents(&full_commandline);
                 assert_eq!(
                     compute_layout(
                         '…',
