@@ -53,7 +53,7 @@ expect_str("bryellow")
 expect_str("cyan")
 expect_str("green")
 expect_str("magenta")
-expect_str("\x1b[31m\x1b[44mred")
+expect_str("\x1b[31;44mred")
 expect_str("white")
 expect_str("yellow")
 expect_str("normal")
